Attribute VB_Name = "mManifest"
Option Explicit

Public Enum LANG_ID
    LANG_NONE = -1
    LANG_NEUTRAL = &H0
    LANG_BULGARIAN = &H2
    LANG_CHINESE = &H4
    LANG_CROATIAN = &H1A
    LANG_CZECH = &H5
    LANG_DANISH = &H6
    LANG_DUTCH = &H13
    LANG_ENGLISH = &H9
    LANG_FINNISH = &HB
    LANG_FRENCH = &HC
    LANG_GERMAN = &H7
    LANG_GREEK = &H8
    LANG_HUNGARIAN = &HE
    LANG_ICELANDIC = &HF
    LANG_ITALIAN = &H10
    LANG_JAPANESE = &H11
    LANG_KOREAN = &H12
    LANG_NORWEGIAN = &H14
    LANG_POLISH = &H15
    LANG_PORTUGUESE = &H16
    LANG_ROMANIAN = &H18
    LANG_RUSSIAN = &H19
    LANG_SLOVAK = &H1B
    LANG_SLOVENIAN = &H24
    LANG_SPANISH = &HA
    LANG_SWEDISH = &H1D
    LANG_TURKISH = &H1F
End Enum

Public Enum SUBLANG_ID
    SUBLANG_NONE = -1
    SUBLANG_NEUTRAL = &H0                       '  language neutral
    SUBLANG_DEFAULT = &H1                       '  user default
    SUBLANG_SYS_DEFAULT = &H2                   '  system default
    SUBLANG_CHINESE_TRADITIONAL = &H1           '  Chinese (Taiwan)
    SUBLANG_CHINESE_SIMPLIFIED = &H2            '  Chinese (PR China)
    SUBLANG_CHINESE_HONGKONG = &H3              '  Chinese (Hong Kong)
    SUBLANG_CHINESE_SINGAPORE = &H4             '  Chinese (Singapore)
    SUBLANG_DUTCH = &H1                         '  Dutch
    SUBLANG_DUTCH_BELGIAN = &H2                 '  Dutch (Belgian)
    SUBLANG_ENGLISH_US = &H1                    '  English (USA)
    SUBLANG_ENGLISH_UK = &H2                    '  English (UK)
    SUBLANG_ENGLISH_AUS = &H3                   '  English (Australian)
    SUBLANG_ENGLISH_CAN = &H4                   '  English (Canadian)
    SUBLANG_ENGLISH_NZ = &H5                    '  English (New Zealand)
    SUBLANG_ENGLISH_EIRE = &H6                  '  English (Irish)
    SUBLANG_FRENCH = &H1                        '  French
    SUBLANG_FRENCH_BELGIAN = &H2                '  French (Belgian)
    SUBLANG_FRENCH_CANADIAN = &H3               '  French (Canadian)
    SUBLANG_FRENCH_SWISS = &H4                  '  French (Swiss)
    SUBLANG_GERMAN = &H1                        '  German
    SUBLANG_GERMAN_SWISS = &H2                  '  German (Swiss)
    SUBLANG_GERMAN_AUSTRIAN = &H3               '  German (Austrian)
    SUBLANG_ITALIAN = &H1                       '  Italian
    SUBLANG_ITALIAN_SWISS = &H2                 '  Italian (Swiss)
    SUBLANG_NORWEGIAN_BOKMAL = &H1              '  Norwegian (Bokma
    SUBLANG_NORWEGIAN_NYNORSK = &H2             '  Norwegian (Nynorsk)
    SUBLANG_PORTUGUESE = &H2                    '  Portuguese
    SUBLANG_PORTUGUESE_BRAZILIAN = &H1          '  Portuguese (Brazilian)
    SUBLANG_SPANISH = &H1                       '  Spanish (Castilian)
    SUBLANG_SPANISH_MEXICAN = &H2               '  Spanish (Mexican)
    SUBLANG_SPANISH_MODERN = &H3                '  Spanish (Modern)
End Enum

'Resource updating (NT/2000/XP only) - Unicode versions
Private Declare Function BeginUpdateResourceNT Lib "kernel32" Alias "BeginUpdateResourceW" (ByVal pFileName As Long, ByVal bDeleteExistingResources As Long) As Long
Private Declare Function UpdateResourceNT Lib "kernel32" Alias "UpdateResourceW" (ByVal hUpdate As Long, ByVal lpType As Long, ByVal lpName As Long, ByVal wLanguage As Integer, lpData As Any, ByVal cbData As Long) As Long
Private Declare Function EndUpdateResourceNT Lib "kernel32" Alias "EndUpdateResourceW" (ByVal hUpdate As Long, ByVal fDiscard As Long) As Long

'Enumerating resources
Private Declare Function EnumResourceNames Lib "kernel32" Alias "EnumResourceNamesA" (ByVal hModule As Long, ByVal lpType As String, ByVal lpEnumFunc As Long, ByVal lParam As Long) As Long
Private Declare Function EnumResourceNamesById Lib "kernel32" Alias "EnumResourceNamesA" (ByVal hModule As Long, ByVal lpType As Long, ByVal lpEnumFunc As Long, ByVal lParam As Long) As Long
Private Declare Function EnumResourceLanguages Lib "kernel32" Alias "EnumResourceLanguagesA" (ByVal hModule As Long, ByVal lpType As Long, ByVal lpName As Long, ByVal lpEnumFunc As Long, ByVal lParam As Long) As Long

Private Const RT_MANIFEST = 24&

Private Declare Function FreeLibrary Lib "kernel32" (ByVal hLibModule As Long) As Long
Private Declare Function LoadLibraryEx Lib "kernel32" Alias "LoadLibraryExA" (ByVal lpLibFileName As String, ByVal hFile As Long, ByVal dwFlags As Long) As Long
Private Declare Function CopyStringA Lib "kernel32" Alias "lstrcpyA" (ByVal NewString As String, ByVal OldString As Long) As Long
Private Declare Function lstrlenA Lib "kernel32" (ByVal lpString As Long) As Long

Dim m_wLangID As Integer
Dim sManifestName As String

Public Function AddManifest(ByVal sExeName As String, _
                            Optional sAppName As String, _
                            Optional sAppDescription As String, _
                            Optional ByVal LANG_XXX As LANG_ID = LANG_NEUTRAL, _
                            Optional ByVal SUBLANG_XXX As SUBLANG_ID = SUBLANG_NEUTRAL) As Boolean
   Dim hModule As Long
   Dim ret As Long, nExtraBytes As Long
   Dim lpName As Long
   Dim sManifest As String
   Dim abManifest() As Byte
   
   hModule = LoadLibraryEx(sExeName, 0, 1)
   If hModule = 0 Then
      MsgBox "Can not load library!", vbCritical, "Error"
      Exit Function
   End If
   sManifestName = ""
   Call EnumResourceNamesById(hModule, RT_MANIFEST, AddressOf ResNamesCallBack, 0)
   If sManifestName <> "" Then
      ret = MsgBox("This file already have manifest resource. Would you like replace it with new one?", vbYesNo + vbExclamation, "Warning")
      If ret = vbNo Then GoTo ErrOut
      GetManifestLanguage hModule, sManifestName
   End If
   FreeLibrary hModule
   Dim hUpdate As Long
   hUpdate = BeginUpdateResourceNT(StrPtr(sExeName), False)
   If hUpdate = 0 Then
      MsgBox "Can not update resources!", vbCritical, "Error"
      GoTo ErrOut
   End If
   
   If sManifestName <> "" Then
      If IsNumeric(sManifestName) Then
         lpName = CLng(sManifestName)
      Else
         lpName = StrPtr(sManifestName)
      End If
      ret = UpdateResourceNT(hUpdate, RT_MANIFEST, lpName, m_wLangID, ByVal 0&, 0)
      Call EndUpdateResourceNT(hUpdate, 0)
      hUpdate = BeginUpdateResourceNT(StrPtr(sExeName), 0)
   End If
   If m_wLangID = 0 Then m_wLangID = MakeLangID(LANG_XXX, SUBLANG_XXX)
   sManifest = CreateManifest(sAppName, sAppDescription)
   nExtraBytes = 4 - (Len(sManifest) Mod 4)
   If nExtraBytes = 4 Then nExtraBytes = 0
   If nExtraBytes Then sManifest = sManifest & String(nExtraBytes, " ")
   abManifest = StrConv(sManifest, vbFromUnicode)
   ret = UpdateResourceNT(hUpdate, RT_MANIFEST, 1, m_wLangID, abManifest(0), UBound(abManifest) + 1)
   Call EndUpdateResourceNT(hUpdate, 0)
   AddManifest = ret
ErrOut:
   If hModule Then FreeLibrary hModule
End Function

Function ResLanguagesCallBack(ByVal hMod As Long, ByVal ResType As Long, ByVal ResName As Long, ByVal wLangID As Integer, ByVal lParam As Long) As Long
    If wLangID Then m_wLangID = wLangID
    ResLanguagesCallBack = 1 'True
End Function

Function ResNamesCallBack(ByVal hMod As Long, ByVal ResType As Long, ByVal ResId As Long, ByVal lParam As Long) As Long
    If (ResId And &HFFFF0000) <> 0 Then
       sManifestName = StrFromPtrA(ResId)
    Else
       sManifestName = CStr(ResId)
    End If
    ResNamesCallBack = (sManifestName = "")
End Function

Private Sub GetManifestLanguage(ByVal hModule As Long, ByVal ResName As String)
   Dim ret As Long
   Dim lngName As Long
   Dim abName() As Byte
   
   If Left(ResName, 1) = "#" Then ResName = Mid(ResName, 2)
   
   If IsNumeric(ResName) Then
      lngName = CLng(ResName)
   Else
      abName = StrConv(ResName & Chr(0), vbFromUnicode)
      lngName = VarPtr(abName(0))
   End If
   ret = EnumResourceLanguages(hModule, RT_MANIFEST, lngName, AddressOf ResLanguagesCallBack, 0)
End Sub

Private Function MakeLangID(ByVal usPrimaryLanguage As LANG_ID, ByVal usSubLanguage As SUBLANG_ID) As Long
    MakeLangID = usSubLanguage * 2 ^ 10 + usPrimaryLanguage
End Function

Private Function CreateManifest(Optional sProgName As String, Optional sProgDesc As String) As String
   Dim s As String
   s = "<?xml version=""1.0"" encoding=""UTF-8"" standalone=""yes""?>"
   s = s & vbCrLf & "<assembly xmlns=""urn:schemas-microsoft-com:asm.v1"" manifestVersion=""1.0"">"
   s = s & vbCrLf & "<assemblyIdentity"
   s = s & vbCrLf & "    name=" & Chr(34) & sProgName & Chr(34)
   s = s & vbCrLf & "    processorArchitecture = ""x86"""
   s = s & vbCrLf & "    version=""1.0.0.0"""
   s = s & vbCrLf & "    type=""win32""/>"
   s = s & vbCrLf & "<description>" & sProgDesc & "</description>"
   s = s & vbCrLf & "<dependency>"
   s = s & vbCrLf & "    <dependentAssembly>"
   s = s & vbCrLf & "        <assemblyIdentity"
   s = s & vbCrLf & "            type=""win32"""
   s = s & vbCrLf & "            name = ""Microsoft.Windows.Common-Controls"""
   s = s & vbCrLf & "            version = ""6.0.0.0"""
   s = s & vbCrLf & "            processorArchitecture = ""x86"""
   s = s & vbCrLf & "            publicKeyToken = ""6595b64144ccf1df"""
   s = s & vbCrLf & "            language = ""*"""
   s = s & vbCrLf & "        />"
   s = s & vbCrLf & "    </dependentAssembly>"
   s = s & vbCrLf & "</dependency>"
   s = s & vbCrLf & "</assembly>"
   CreateManifest = s
End Function

Private Function StrFromPtrA(ByVal lpszA As Long, Optional nSize As Long = 0) As String
   Dim s As String, bTrim As Boolean
   If nSize = 0 Then
      nSize = lstrlenA(lpszA)
      bTrim = True
   End If
   s = String(nSize, Chr$(0))
   CopyStringA s, ByVal lpszA
   If bTrim Then s = TrimNull(s)
   StrFromPtrA = s
End Function

Private Function TrimNull(ByVal str As String) As String
    If InStr(str, Chr$(0)) > 0& Then
        TrimNull = Left$(str, InStr(str, Chr$(0)) - 1&)
    Else
        TrimNull = str
    End If
End Function
